import { QueueModel } from 'src/modules/queue/data/models/queue.model';
import { CustomerQueueManager } from './customer-queue.manager';
import { QueueCondition } from '../formula/queue-condition.formula';

export class CustomerQueueDetailManager extends CustomerQueueManager {
  private queues: QueueModel[] = [];
  currentQueues(queues: QueueModel[]) {
    this.queues = queues;
  }
  get data() {
    const queueCondition = new QueueCondition(this.queues);
    return this.tickets.map((ticket) => {
      return {
        id: ticket.id,
        code: ticket.code,
        customer: ticket.customer,
        phone: ticket.phone,
        date: ticket.date,
        summary: {
          total_activities: this.totalActivities(ticket),
          total_tickets: this.totalTickets(ticket),
          total_used: this.totalUsedTickets(ticket),
          total_queue: this.totalQueueTickets(ticket),
        },
        items: ticket.items.map((item) => {
          const queueItem = item.item.item_queue ?? item.item;
          return {
            id: item.item_id,
            item_queue_id: item.id,
            title: queueItem.name,
            image_url: item.item.image_url,
            summary: {
              total_tickets: item.qty,
              total_used: this.totalUsedItems(item),
              total_queue: this.totalQueueItems(item),
            },
            queue: item.queue
              .sort((a, b) => b.time - a.time)
              .map((q) => {
                return {
                  code: q.code,
                  qty: q.qty,
                  time: this.toTime(q.time),
                  status: q.status,
                };
              }),
            queue_condition: queueCondition.condition(queueItem.id),
          };
        }),
      };
    });
  }
}
